﻿----------------------------------------
-- Epic Gem List v 0.2, Copyright 2010 Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	Added blue cataclysm gems with test on first load
-- 0.3  Added JC only gems, cataclysm green and WotLK blue gems
-- 0.4	fixed loading of missing gems on first start (delayed GetItemInfo)
-- 0.5	added french translation, thanks to Araldwenn
-- 1.0	most translations from GlobalStrings.lua, quality and level selection, source cleanup, minor fixes
-- 1.1	new design, more detailed selections
-- 1.2	included BC gems
-- 1.3	small bugfixes (2 itemids), added debug mode (/egl debug)
-- 1.4b	added caching DB and options per char in SavedVariables
-- 1.5	show MarketPrice from Auctioneer, if installed
-- 1.6	new custom frame and menu design
-- 1.7	show details in result list
-- 1.80	temporary fix for not loading cataclysm gems
-- 1.82	saves favorite gems per char, fixed JC gem stats
-- 1.90	added recommendations for each class / spec
-- 2.00	added LibVersionCheck, shows all cataclysm gems
-- 2.01	added minimap button (for non LDB users)
-- 2.02	toggle minimap button. show gem colors in list
-- 2.03	added missing cataclysm gems, some small fixes
-- 2.04	temporarily removed auction house prices due to lua errors
-- 2.05	hopefully fixed delay on login
-- 2.06	some more tweaks which should speed up loading
-- 2.10 added new meta gems for 4.0.6, some classes changed
-- 2.11 added Vivid Dream Emerald
-- 2.12 added Reckless Hessonite
-- 2.20 added guild crafters for gems
-- 2.21 added config window
-- 2.22 small fixes
-- 2.23 toc change and filter fixed
-- 2.24 design fixed and 2 missing gems added
-- 2.30 added epic gems 4.3.0, some ugly fixes
-- 2.31 changed toc for 5.0.5 (will add data later)
-- 3.00 added all pandarian gems, removed guild scanner
-- 3.01 bugfixes
-- 3.02 bugfixes, removes libversioncheck, fixed toc
-- 3.03 changed TOC for 6.0.2, updated libs, added legendary gems
-- 3.04 removed debugging output
-- 4.00 replaced stats for 6.02 (exp, hit, ...), code cleanup, bugfixes
-- 4.01 added WoD gems
-- 4.02 toc change for 6.1
-- 5.00 updated for Legion, new gems, changed sockets and stats, Raidframe1 Bugfix

----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------
EGL_TOPVER = "5";
EGL_SUBVER = "00";

EGL_VER = EGL_TOPVER.."."..EGL_SUBVER;
EGL_VERSION = "ver "..EGL_VER.." by Nekudotayim @ Shattrath"

EGL_SOCKETSELECT = _G["COLOR"];
EGL_STATSELECT = _G["STATS_LABEL"];
EGL_CLOSE = _G["CLOSE"];
EGL_JC = EGL_GemLocaleText["jc"];

EGL_SCAN_FRAME_TITLE = "Epic Gem List";
EGL_SCAN_FRAME_SUBTITLE = "Retrieving Gem Data from server ...";
EGL_SCAN_FRAME_ABORT = "cancel";
EGL_SCAN_FRAME_INITIALIZING = "Initializing scan ...";
EGL_QUALITY = _G["QUALITY"];
EGL_ITEMLEVEL = _G["LEVEL"];

EGL_SHOWFAVORITES = EGL_GemLocaleText["fav"];
EGL_HIDEFAVORITES = _G["ALL"];  
EGLBUILD = nil;
EGLDBVER = nil;
egl_rs = nil;
EGL_DB_LOADED = nil;
tscurrent = 0;
tsmax = 0;

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local waitTable = {};
local waitFrame = nil;
local EGL_Scanned_Gems;
local MY_ADDON_NAME = "EpicGemList";
local MY_ADDON_VERSION = EGL_TOPVER..EGL_SUBVER;

-- INITIALIZE UNSET VARS ------------------------------------------------

if(EGL_Options == nil) then  -- saving options per char
    EGL_Options = {};
end

if(EGLDB == nil) then -- saving gem data per account
    EGLDB = {};
end

if (EGL_FAV == nil) then -- saving favorites per char
	EGL_FAV = {};
end


if ( EGL_Options["SelectedStat"]  == nil ) then
    EGL_Options["SelectedStat"] = 0;
end

if ( EGL_Options["SelectedSlot"] == nil ) then
    EGL_Options["SelectedSlot"] = 0;
end

if ( EGL_Options["SelectedQualiOp"] == nil ) then
    EGL_Options["SelectedQualiOp"] = 1;
end

if ( EGL_Options["SelectedQuali"] == nil ) then
    EGL_Options["SelectedQuali"] = 2;
end

if ( EGL_Options["SelectedLevelOp"] == nil ) then
    EGL_Options["SelectedLevelOp"] = 1;
end

if ( EGL_Options["SelectedLevel"] == nil ) then
    EGL_Options["SelectedLevel"] = 6;
end

if (EGL_Options["jc"] == nil ) then
	EGL_Options["jc"] = false;
	EGL_Options["jc_cata"] = false;
	EGL_Options["jc_panda"] = false;
end

if (EGL_Options["jc"] and EGL_Options["cataclysm"]) then
	EGL_Options["jc_cata"] = true;
end
if (EGL_Options["jc"] == false) then
	EGL_Options["jc_cata"] = false
end

if ( EGL_Options["buy"]  == nil ) then
    EGL_Options["buy"] = false;
end

if ( EGL_Options["loot"]  == nil ) then
    EGL_Options["loot"] = false;
end

if ( EGL_Options["details"]  == nil ) then
    EGL_Options["details"] = false;
end

if ( EGL_Options["favorites"] == nil ) then
	EGL_Options["favorites"] = false;
end

if (EGL_Options["debug"] == nil) then
	EGL_Options["debug"] = false
end

if (EGL_Options["minimap"] == nil) then
	EGL_Options["minimap"] = true
end

if (EGL_Options["gc"] == nil) then
	EGL_Options["gc"] = true
end

if (EGL_Scanned_Gems == nil) then
	EGL_Scanned_Gems = 0;
end
if (egl_scan_state == nil) then
	egl_scan_state = 0;
end
if (EGL_GemCount == nil) then
	EGL_GemCount = 0;
end
if (egl_shown_gems == nil) then
	egl_shown_gems = 0;
end


-- MISC FUNCTIONS ------------------------------------------------

---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
end

---- Delayed script execution ----
function EGL_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

--- counting table rows with every index format
function egl_tcount(tab)
   local n = #tab
   if (n == 0) then
     for _ in pairs(tab) do
       n = n + 1
     end
   end
   return n
end

--- Market Price from Auctioneer, if installed
function EGL_MarketPrice(itemID)
	if not itemID then
		if EGL_Options["debug"] then
			return "noItemID";
		end;
		return "";
	end
	aucinstalled = false;
	itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(itemID);
	if AucAdvanced then -- Auctioneer
		sellPrice, numSeen = AucAdvanced.API.GetMarketValue(itemLink);
		aucinstalled = true;
	end;
	if sellPrice == nil then
		return "";
	else
		sellPrice = tonumber(sellPrice);
	end
	if aucinstalled then
		if (sellPrice ~= nil) and (sellPrice > 0) then
			local g = math.floor(sellPrice/10000);
			local GSC_GOLD = "ff807000"
			local digits = math.floor(math.log10(sellPrice)+1)
			if digits < 9 then
				gsc = string.format("|c%s%5d|r", GSC_GOLD, g)
			else
				gsc = string.format("|c%s%2.1fk|r", GSC_GOLD, g/1000)
			end
			if g == 0 then
				return "";
			end
			return gsc;
		end
	end
	return "";
end

--- scan 2. line of tooltip ---
function EGL_GetToolTipStats(itemid)
	ScanTooltip:ClearLines();
	ScanTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
    ScanTooltip:SetHyperlink("item:"..itemid..":0:0:0");
	itemStats = "";
	for i=2,ScanTooltip:NumLines() do
		local mytext=_G["ScanTooltipTextLeft"..i];
		local text=mytext:GetText();
		if i == 2 then
			itemStats = text; -- use first line on default
		end
		if string.find(text, "+") then
			itemStats = text; -- use line with + in text for stats
		end
 	end
	return itemStats;
end


EGLAddon = {}


-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Gem List", {
    type = "launcher",
	label = "EpicGemList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EGL_Form:IsVisible() then
		        EGL_Form:Hide()
		    else
		        EGL_Form:Show()
	        end
        elseif msg == "RightButton" then
			if not EGL_Config:IsVisible() then
				EGL_Config:Show()
			else
				EGL_Config:Hide()
			end
		end
    end,
	icon = "Interface\\AddOns\\EpicGemList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Gem List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EGL_SlashCommandHandler(msg)
	if (msg == "debug") then
		if EGL_Options["debug"] then
			EGL_Options["debug"] = false;
			out("EGL: Debugging disabled");
		else
			EGL_Options["debug"] = true;
			if (EGL_Form:IsVisible() == false) then
				EGL_Form:Show()
			end
			
			out("EGL: Debugging enabled");
		end
	elseif (msg == "minimap") then
		if EGL_MinimapButton:IsVisible() then
			EGL_Options["minimap"] = false;
			EGL_MinimapButton:Hide();
		else
			EGL_Options["minimap"] = true;
			EGL_MinimapButton:Show();
		end
	elseif (msg == "scan") then
		EGL_ForceScan();
		if (EGL_Form:IsVisible() == false) then
			EGL_Form:Show();
		end
	elseif (msg == "show") then
		EGL_Form:Show()
	elseif (msg == "hide") then
		EGL_Form:Hide()
	elseif (msg == "config") then
		if not EGL_Config:IsVisible() then
			EGL_Config:Show()
		else
			EGL_Config:Hide()
		end
	else
		out("|cFFFFFF00Epic Gem List: ");
		out("|cFFFFFF00("..EGL_VERSION..")");
		out("|cFF00FF00/egl show |cFFFFFFFF[show main window]");
		out("|cFF00FF00/egl hide |cFFFFFFFF[hide main window]");
		out("|cFF00FF00/egl config |cFFFFFFFF[show config window]");
		out("|cFF00FF00/egl scan |cFFFFFFFF[rescan and rebuild gem database]");
		out("|cFF00FF00/egl minimap |cFFFFFFFF[show/hide minimap button]");
		out("|cFF00FF00/egl debug |cFFFFFFFF[show/hide debugging messages]");
	end
end

-- ON LOAD ------------------------------------------------

function EGL_OnLoad(this)
	EGL_Form:RegisterEvent("VARIABLES_LOADED");
	eglf = _G["EGL_Form"];
	SLASH_EGL1 = "/epicgemlist";
	SLASH_EGL2 = "/egl";
	SlashCmdList["EGL"] = function(msg)
		EGL_SlashCommandHandler(msg);
	end
	if EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
	EGL_Options = {};
	EGL_Options["SelectedSlot"] = 0;
	EGL_Options["SelectedStat"] = 0;
	EGL_Options["SelectedQualiOp"] = 1;
	EGL_Options["SelectedQuali"] = 2;
	EGL_Options["SelectedLevelOp"] = 1;
	EGL_Options["SelectedLevel"] = 6;
	EGL_Options["SelectedSpec"] = 0;
	EGL_Options["debug"] = false;
	EGL_Options["details"] = false;
	EGL_Options["favorites"] = false;
	EGL_Options["cataclysm"] = true;
	EGL_Options["gc"] = false;
	EGL_Scanned_Gems = 0;
	local version, build, builddate, tocversion = GetBuildInfo();
end

-- ON SHOW ------------------------------------------------

function EGL_OnShow()
	if EGL_Options["jc"] then
		CheckButtonJC:SetChecked(true);
	else
		CheckButtonJC:SetChecked(false);
	end
	if EGL_Options["details"] then
		CheckButtonDetails:SetChecked(true);
	else
		CheckButtonDetails:SetChecked(false);
	end	
	local favb = _G["EGL_FavButton"];
	if EGL_Options["favorites"] == true then
		favb:SetText(EGL_HIDEFAVORITES);
	else
		favb:SetText(EGL_SHOWFAVORITES);
	end	
	for i = 1, 15 do
		item = _G["EGLItem_"..i];
		item:Hide();
	end
	EGL_ScrollBar_Update();
end

-- ON EVENT ------------------------------------------------

function EGL_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		-- Nasty "hack", load Blizzard_Calendar if not loaded,
		-- for it to secure init 24 dropdown menu buttons,
		-- to avoid action blocked by tainting
		if not IsAddOnLoaded("Blizzard_Calendar") then
			LoadAddOn("Blizzard_Calendar")
		end
		local version, build, builddate, tocversion = GetBuildInfo();
		local mybuild = version..build;
		EGL_DB_LOADED = true;
		EGL_Options["cataclysm"] = true;
		if (EGLBUILD == nil) or (EGLDBVER == nil) then
			EGLBUILD = mybuild;
			EGLDBVER = EGL_VER;
			out("EGL: Scanning gems for local database ...");
			EGL_GemScan();
		end
		if (EGLBUILD ~= mybuild) or (EGLDBVER < EGL_VER) then
			EGLBUILD = mybuild;
			EGLDBVER = EGL_VER;
			out("EGL: Scanning gems for new WOW-Client "..version.." (Build "..build.." / "..builddate..") and EGL "..EGL_VER);
			EGL_GemScan();
		end
		EGL_Options["cataclysm"] = true;
		if EGL_Options["jc"] then
			CheckButtonJC:SetChecked(true);
		else
			CheckButtonJC:SetChecked(false);
		end
		if EGL_Options["minimap"] == true then
			EGL_MinimapButton:Show()
			CheckButtonConfig1:SetChecked(true);
		else
			EGL_MinimapButton:Hide()
			CheckButtonConfig1:SetChecked(false);
		end
		if EGL_Options["debug"] == true then
			CheckButtonConfig5:SetChecked(true);
		else
			CheckButtonConfig5:SetChecked(false);
		end
		EGL_SlotDropDown_Init();
		EGL_StatDropDown_Init();
		EGL_ScrollBar_Update();
	end
end


-- GET GEM INFO ------------------------------------------------

function EGL_GemInfo(value)
	if EGL_DB_LOADED then
		if EGLDB[value] ~= nil then 
			local iteminfo = EGLDB[value];
			if iteminfo[6] == nil then
				iteminfo[6] = EGL_GetToolTipStats(value);
				EGLDB[value] = { iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6] }
			end
			return iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6];
		else
			EGL_GetGemInfo(value);
			return false;
		end
	else
		if EGL_Options["debug"] then
			out("EGL DEBUG: EGLDB not yet loaded");
		end
		return false;
	end
end
-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EGL_Help_OnEnter(this)
	local entries = table.getn(EGL_Help);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Help[i]);
	end
    EGLTooltip:Show();
end

function EGL_Help_OnLeave()
  EGLTooltip:Hide();
end
-- FAV BUTTON ------------------------------------------------

function EGL_Fav_OnEnter(this)
	local entries = table.getn(EGL_Fav);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Fav[i]);
	end
    EGLTooltip:Show();
end

function EGL_Fav_OnLeave()
  EGLTooltip:Hide();
end

function EGL_Fav_OnClick(this)
	local favb = _G["EGL_FavButton"];
	local recb = _G["EGL_RecButton"];
	if EGL_Options["favorites"] == true then
		EGL_Options["favorites"] = false;
		favb:SetText(EGL_SHOWFAVORITES);
	else
		EGL_Options["favorites"] = true;
		favb:SetText(ALL);
	end
	EGL_ScrollBar_Update();
end


-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EGLItem_OnEnter(this)
local addmembers = "GuildCrafters: ";

	EGLTooltip:ClearLines();
	local id = this:GetID()+ FauxScrollFrame_GetOffset(EGL_ScrollBar);  
	if id ~= nil then
		local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);
		EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
		EGLTooltip:SetHyperlink("item:"..EGL[id]..":0:0:0");
		EGLTooltip:AddLine("Item Level: "..itemLevel..", ID: "..EGL[id], 0.7,0.7,1);
		EGLTooltip:Show();
	end 
end

function EGLItem_OnLeave()
  SELECTEDITEM = 0;
  EGLTooltip:Hide();
end

function EGLItem_OnClick(this)
	local EGL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EGL_ScrollBar);
	local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);

	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and itemName) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
	if IsControlKeyDown() then
		if EGL_FAV[EGL[id]] ~= nil then
			EGL_FAV[EGL[id]] = nil;
			EGL_ScrollBar_Update();
		else
			EGL_FAV[EGL[id]] = true;
			EGL_ScrollBar_Update();
		end
	end
end


-- RESULT LIST ------------------------------------------------



function EGL_HasNotStat(egltable,value)
	if EGL_Options["SelectedStat"] ~= 0 and 
	 (egltable[value]["stat1"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat2"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat3"] ~= EGL_Options["SelectedStat"]) then
		return true;
	else
		return false;
	end

end

function EGL_IsNotSlot(egltable,value)
	if EGL_Options["SelectedSlot"] == 1 and egltable[value]["meta"] == false then
		return true;
	else
		return false;
	end
end

function EGL_IsNotQuali(value)
	if EGL_Options["SelectedQuali"] ~= 0 then
		local itemName, itemLink, itemRarity, itemLevel, itemTexture = EGL_GemInfo(value);
		if itemRarity == nil then
			return false;
		end
		if EGL_Options["SelectedQualiOp"] == 0 then
			if (itemRarity ~= (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 1 then
			if (itemRarity < (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 2 then
			if (itemRarity > (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		end	
	end
	return false;
end

function EGL_ShowLevel(selected,required)
	if EGL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EGL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentgem;    
    EGL = {}; 
    currentgem = 1;
	egl_shown_gems = 0;
	
	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2) then -- ALL, WotLK
		for value,_ in pairs(EGL_Data) do    
			additem = true;
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
	
	if ((EGL_Options["cataclysm"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cataclysm
		for value,_ in pairs(EGL_Data_Cata) do    
			additem = true;	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end

	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],4) then -- ALL, Panda
		for value,_ in pairs(EGL_Data_Panda) do    
			additem = true;
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Panda,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Panda,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end

	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],5) then -- ALL, WoD
		for value,_ in pairs(EGL_Data_WOD) do    
			additem = true;
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_WOD,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_WOD,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end

	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],6) then -- ALL, Legion
	for value,_ in pairs(EGL_Data_Legion) do
		additem = true;
		if EGL_Options["favorites"] then
			if EGL_FAV[value] == nil then
				additem = false;
			end
		end
		if EGL_IsNotQuali(value) then
			additem = false;
		elseif EGL_HasNotStat(EGL_Data_Legion,value) then
			additem = false;
		elseif EGL_IsNotSlot(EGL_Data_Legion,value) then
			additem = false;
		elseif (EGL_GemInfo(value) == false) then
			additem = false;
		end
		if additem == true then
			tinsert(EGL, value);
		end
		currentgem = currentgem + 1;
	end
	end

	if ((EGL_Options["jc"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2))) then	-- jc + ALL, WotLK
		for value,_ in pairs(EGL_Data_JC) do    
			additem = true;	    	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end

	if ((EGL_Options["jc_cata"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cata
		for value,_ in pairs(EGL_Data_Cata_JC) do    
			additem = true;	    	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end	

	if ((EGL_Options["jc"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],4))) then	-- if ALL, Panda
		for value,_ in pairs(EGL_Data_Panda_JC) do    
			additem = true;	    	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Panda_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Panda_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end	
	
	
	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],1) then -- ALL, BC
		for value,_ in pairs(EGL_Data_BC) do    
			additem = true;	  
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_BC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_BC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
    table.sort(EGL, EGL_SortResults);
    EGL_ShowGemList();    
end

function EGL_ScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc)
	if EGL_Options["details"] then
		lineheight = 32;
	end
	FauxScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc);
end


function EGL_SortResults(a, b)
	local iteminfoA = EGL_GemInfo(a);
	local iteminfoB = EGL_GemInfo(b);
    local itemNameA, itemLinkA, itemQualityA, itemLevelA = EGL_GemInfo(a);
    local itemNameB, itemLinkB, itemQualityB, itemLevelB = EGL_GemInfo(b);
	
	if EGL_Options["debug"] then
		return a < b;
	end
	if ((iteminfoA ~= nil) and (iteminfoB ~= nil)) then
		if itemLevelA ~= itemLevelB then	-- Sort by item level first
			return itemLevelA > itemLevelB;
		end
		if itemQualityA ~= itemQualityB then	-- Sort by item quality second
			return itemQualityA > itemQualityB;
		end
		if itemNameA and itemNameB then		-- if same item level, sort by name
			return itemNameA < itemNameB;
		end
	end
	return a < b;						-- sort by id, if no name
end


function EGL_ShowGemList()  
	local i, item, itemm, itemn, itemi, itemf, itemr, itemb, itemy;
	local sg = 15;
    MaxItems = egl_tcount(EGL);
	if EGL_Options["details"] then
		lineheight = 32;
		sg = 7;
	else
		lineheight = 16;
		sg = 15;
	end
    FauxScrollFrame_Update(EGL_ScrollBar,MaxItems,sg,lineheight);
    for i=1,sg do
		-- get global button names from xml script
		item = _G["EGLItem_"..i];
		itemn = _G["EGLItem_"..i.."_Name"];
		itemi = _G["EGLItem_"..i.."_Icon"];
		itemf = _G["EGLItem_"..i.."_Fav"];
		itemp = _G["EGLItem_"..i.."_MarketPrice"];
		iteme = _G["EGLItem_"..i.."_Extra"];
		
        lineplusoffset = i + FauxScrollFrame_GetOffset(EGL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			-- get gem data from local DB or load from server if not available
			local itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats = EGL_GemInfo(EGL[lineplusoffset]);
			
			if (itemStats == nil) or (itemStats == "") then
				itemStats = EGL_GetToolTipStats(EGL[lineplusoffset]);
			end
			if itemName then
				if iteme ~= nil then
					-- show detailed info in result list ?
					if EGL_Options["details"] then
						item:SetHeight(30);
						iteme:SetText(itemStats);
						iteme:Show();
					else
						item:SetHeight(15);
						iteme:SetText("");
						iteme:Hide();
					end
				end

				if itemn ~= nil then
					if EGL_Options["debug"] then
						itemn:SetText(EGL[lineplusoffset]..ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
					else
						itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
					end
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if itemp ~= nil then
					eglprice = EGL_MarketPrice(EGL[lineplusoffset]);
					if eglprice then
						itemp:SetText(eglprice);
					end
				end
				if itemf ~= nil then
					if EGL_FAV[EGL[lineplusoffset]] ~= nil then
						itemf:SetTexture("Interface/GROUPFRAME/UI-Group-MasterLooter");
						itemf:Show();
					else
						itemf:SetTexture("");
						itemf:Hide();
					end
				end
				if item ~= nil then
					item:Show();
				end

			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EGL_GEMCOUNT"];
	if itemm ~= nil then
		local EGL_AUSWAHL = "";
		if EGL_Options["favorites"] then
			EGL_AUSWAHL = " ("..EGL_SHOWFAVORITES..")";
		end
		itemm:SetText(MaxItems.." gems"..EGL_AUSWAHL..":");
	end
end

-- INITIAL GEM SCAN ------------------------------------------------

function EGL_GetGemInfoDelayed(value)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		local itemStats = EGL_GetToolTipStats(value);
		EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };		
		EGL_ScrollBar_Update();
		if EGL_Options["debug"] then
			out("EGL DEBUG: Added delayed Item ID"..value);
		end
	else
		out("EGL DEBUG: Could not retrieve ItemID "..value.." from server, please report to addon author");
	end
end


function EGL_GetGemInfo(value)
	-- REQUEST FROM SERVER (should be ONLY this line)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		EGL_Scanned_Gems = EGL_Scanned_Gems + 1;
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning Tooltip for "..itemName);
		end
		local itemStats = EGL_GetToolTipStats(value);
		EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };
		return true;
	else
		EGL_wait(10,EGL_GetGemInfoDelayed,value);
		failed = failed + 1;
		if EGL_Options["debug"] then
			out("EGL DEBUG: Failed to get ItemID "..value);
		end
		return false;
	end
end

function EGL_GemScan()
	EGLDB = {};	
	EGL_Scanned_Gems = 0;
	out("EGL: Scanning gems ...")
	if EGL_Options["cataclysm"] then
		for value,_ in pairs(EGL_Data_Cata) do
			if EGL_Options["debug"] then
				out("EGL Debug: Scanning Cataclysm gems");
			end
			EGL_GetGemInfo(value);
		end
		for value,_ in pairs(EGL_Data_Cata_JC) do
			if EGL_Options["debug"] then
				out("EGL Debug: Scanning Cataclysm JC gems");
			end
			EGL_GetGemInfo(value);
		end				
	end
	for value,_ in pairs(EGL_Data) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning WotLK gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_JC) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning WotLK JC gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_BC) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning BC gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_Panda) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning Pandaria gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_WOD) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning WoD gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_Legion) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning Legion gems");
		end
		EGL_GetGemInfo(value);
	end
	EGL_FinalizeScan();
end

function EGL_AbortScan()
	egl_scan_state=0;
end

function EGL_ForceScan()
	egl_scan_state = 0;
	EGL_GemScan();
end

function EGL_FinalizeScan()
	egl_scan_state=1;
	out("EGL: Scanned "..EGL_Scanned_Gems.." gems.");
	if (failed > 0) then
		out("EGL: Missing "..failed.." gems will be added in a few seconds ...");
		failed = 0;
	end
	EGL_ScrollBar_Update();
end


